/**
 * Copyright (C) 2009 STMicroelectronics
 *
 * This file is part of "Mind Compiler" is free software: you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: mind-members@lists.minalogic.net
 *
 * Authors: Matthieu Leclercq, Ali Erdem Ozcan
 * Contributors: 
 */

package org.objectweb.fractal.mind.adl;

import org.antlr.stringtemplate.StringTemplateGroupLoader;
import org.objectweb.fractal.mind.BasicInputResourceLocator;
import org.objectweb.fractal.mind.InputResourceLocator;
import org.objectweb.fractal.mind.adl.factory.FactoryGraphCompiler;
import org.objectweb.fractal.mind.adl.generic.GenericDefinitionNameSourceGenerator;
import org.objectweb.fractal.mind.adl.idl.IDLDefinitionSourceGenerator;
import org.objectweb.fractal.mind.adl.implementation.BasicImplementationLocator;
import org.objectweb.fractal.mind.adl.implementation.ImplementationLocator;
import org.objectweb.fractal.mind.adl.interfaces.CollectionInterfaceDefinitionSourceGenerator;
import org.objectweb.fractal.mind.adl.membrane.MembraneSourceGenerator;
import org.objectweb.fractal.mind.compilation.BasicCompilationCommandExecutor;
import org.objectweb.fractal.mind.compilation.CompilationCommandExecutor;
import org.objectweb.fractal.mind.compilation.CompilerWrapper;
import org.objectweb.fractal.mind.compilation.gcc.GccCompilerWrapper;
import org.objectweb.fractal.mind.idl.IDLBackendFactory;
import org.objectweb.fractal.mind.idl.IDLLoader;
import org.objectweb.fractal.mind.idl.IDLLoaderChainFactory;
import org.objectweb.fractal.mind.idl.IDLVisitor;
import org.objectweb.fractal.mind.idl.st.IDLLoaderASTTransformer;
import org.objectweb.fractal.mind.io.BasicOutputFileLocator;
import org.objectweb.fractal.mind.io.OutputFileLocator;
import org.objectweb.fractal.mind.preproc.BasicMPPWrapper;
import org.objectweb.fractal.mind.preproc.MPPWrapper;
import org.objectweb.fractal.mind.st.BasicASTTransformer;
import org.objectweb.fractal.mind.st.STLoaderFactory;
import org.objectweb.fractal.mind.st.STNodeFactoryImpl;
import org.objectweb.fractal.mind.st.StringTemplateASTTransformer;

public final class ADLBackendFactory {
  private ADLBackendFactory() {
  }

  public static final DefinitionSourceGenerator newDefinitionSourceGenerator() {
    final IDLLoader idlLoader = IDLLoaderChainFactory.newLoader();
    final BasicInputResourceLocator inputResourceLocator = new BasicInputResourceLocator();
    final BasicOutputFileLocator outputFileLocator = new BasicOutputFileLocator();

    final BasicASTTransformer astTransformer = new BasicASTTransformer();
    astTransformer.nodeFactoryItf = new STNodeFactoryImpl();

    final StringTemplateGroupLoader stcLoader = STLoaderFactory.newSTLoader();
    final IDLVisitor idlCompiler = IDLBackendFactory.newIDLCompiler(idlLoader,
        inputResourceLocator, outputFileLocator, astTransformer, stcLoader);

    return newDefinitionSourceGenerator(inputResourceLocator,
        outputFileLocator, idlLoader, idlCompiler, astTransformer, stcLoader);
  }

  public static final DefinitionSourceGenerator newDefinitionSourceGenerator(
      final InputResourceLocator inputResourceLocator,
      final OutputFileLocator outputFileLocator, final IDLLoader idlLoader,
      final IDLVisitor idlCompiler,
      final StringTemplateASTTransformer astTransformer,
      final StringTemplateGroupLoader stcLoader) {

    DefinitionSourceGenerator definitionSourceGenerator;
    final CollectionInterfaceDefinitionSourceGenerator cidsg = new CollectionInterfaceDefinitionSourceGenerator();
    final DefinitionSourceGeneratorDispatcher dsgd = new DefinitionSourceGeneratorDispatcher();
    final DefinitionHeaderSourceGenerator dhsg = new DefinitionHeaderSourceGenerator();
    final DefinitionIncSourceGenerator disg = new DefinitionIncSourceGenerator();
    final ImplementationHeaderSourceGenerator ihsg = new ImplementationHeaderSourceGenerator();
    final DefinitionMacroSourceGenerator dmsg = new DefinitionMacroSourceGenerator();
    final MembraneSourceGenerator msg = new MembraneSourceGenerator();
    final IDLDefinitionSourceGenerator idsg = new IDLDefinitionSourceGenerator();
    final GenericDefinitionNameSourceGenerator gdnsg = new GenericDefinitionNameSourceGenerator();

    definitionSourceGenerator = cidsg;
    cidsg.clientSourceGeneratorItf = dsgd;
    dsgd.visitorsItf.put("header", dhsg);
    dsgd.visitorsItf.put("include", disg);
    dsgd.visitorsItf.put("impl", ihsg);
    dsgd.visitorsItf.put("macro", dmsg);
    dsgd.visitorsItf.put("membrane", msg);
    dsgd.visitorsItf.put("idl", idsg);
    dsgd.visitorsItf.put("generic-names", gdnsg);

    dhsg.inputResourceLocatorItf = inputResourceLocator;
    disg.inputResourceLocatorItf = inputResourceLocator;
    ihsg.inputResourceLocatorItf = inputResourceLocator;
    dmsg.inputResourceLocatorItf = inputResourceLocator;
    msg.inputResourceLocatorItf = inputResourceLocator;

    dhsg.outputFileLocatorItf = outputFileLocator;
    disg.outputFileLocatorItf = outputFileLocator;
    ihsg.outputFileLocatorItf = outputFileLocator;
    dmsg.outputFileLocatorItf = outputFileLocator;
    msg.outputFileLocatorItf = outputFileLocator;
    gdnsg.outputFileLocatorItf = outputFileLocator;

    final IDLLoaderASTTransformer ilat = new IDLLoaderASTTransformer();
    ilat.clientIDLLoaderItf = idlLoader;
    ilat.astTransformerItf = astTransformer;

    idsg.idlLoaderItf = ilat;
    idsg.idlCompilerItf = idlCompiler;

    dhsg.templateGroupLoaderItf = stcLoader;
    disg.templateGroupLoaderItf = stcLoader;
    dmsg.templateGroupLoaderItf = stcLoader;
    msg.templateGroupLoaderItf = stcLoader;

    return definitionSourceGenerator;
  }

  public static DefinitionCompiler newDefinitionCompiler() {
    final IDLLoader idlLoader = IDLLoaderChainFactory.newLoader();
    final BasicInputResourceLocator inputResourceLocator = new BasicInputResourceLocator();
    final BasicOutputFileLocator outputFileLocator = new BasicOutputFileLocator();
    final ImplementationLocator implementationLocator = new BasicImplementationLocator();

    final BasicASTTransformer astTransformer = new BasicASTTransformer();
    astTransformer.nodeFactoryItf = new STNodeFactoryImpl();

    final StringTemplateGroupLoader stcLoader = STLoaderFactory.newSTLoader();

    final IDLVisitor idlCompiler = IDLBackendFactory.newIDLCompiler(idlLoader,
        inputResourceLocator, outputFileLocator, astTransformer, stcLoader);
    final DefinitionSourceGenerator definitionSourceGenerator = newDefinitionSourceGenerator(
        inputResourceLocator, outputFileLocator, idlLoader, idlCompiler,
        astTransformer, stcLoader);
    final CompilerWrapper compilerWrapper = new GccCompilerWrapper();
    final MPPWrapper mppWrapper = new BasicMPPWrapper();
    return newDefinitionCompiler(definitionSourceGenerator,
        implementationLocator, outputFileLocator, compilerWrapper, mppWrapper);
  }

  public static DefinitionCompiler newDefinitionCompiler(
      final DefinitionSourceGenerator definitionSourceGenerator,
      final ImplementationLocator implementationLocator,
      final OutputFileLocator outputFileLocator,
      final CompilerWrapper compilerWrapper, final MPPWrapper mppWrapper) {

    DefinitionCompiler definitionCompiler;
    final BasicDefinitionCompiler bdc = new BasicDefinitionCompiler();

    definitionCompiler = bdc;
    bdc.definitionSourceGeneratorItf = definitionSourceGenerator;
    bdc.implementationLocatorItf = implementationLocator;
    bdc.outputFileLocatorItf = outputFileLocator;
    bdc.compilerWrapperItf = compilerWrapper;
    bdc.mppWrapperItf = mppWrapper;

    return definitionCompiler;
  }

  public static GraphCompiler newGraphCompiler(
      final InputResourceLocator inputResourceLocator,
      final ImplementationLocator implementationLocator,
      final OutputFileLocator outputFileLocator,
      final CompilerWrapper compilerWrapper, final MPPWrapper mppWrapper,
      final DefinitionCompiler definitionCompiler,
      final StringTemplateGroupLoader stcLoader) {

    // Instance source generator
    InstanceSourceGenerator instanceSourceGenerator;
    final BasicInstanceSourceGenerator bisg = new BasicInstanceSourceGenerator();

    instanceSourceGenerator = bisg;
    bisg.inputResourceLocatorItf = inputResourceLocator;
    bisg.outputFileLocatorItf = outputFileLocator;

    bisg.templateGroupLoaderItf = stcLoader;

    // Instance compiler
    InstanceCompiler instanceCompiler;
    final BasicInstanceCompiler bic = new BasicInstanceCompiler();

    instanceCompiler = bic;
    bic.instanceSourceGeneratorItf = instanceSourceGenerator;

    bic.inputResourceLocatorItf = inputResourceLocator;
    bic.compilerWrapperItf = compilerWrapper;
    bic.mppWrapperItf = mppWrapper;
    bic.outputFileLocatorItf = outputFileLocator;

    // graph compiler
    final BasicGraphCompiler bgc = new BasicGraphCompiler();
    final FactoryGraphCompiler fgc = new FactoryGraphCompiler();
    final BasicGraphLinker bgl = new BasicGraphLinker();
    final GraphCompiler graphCompiler = bgl;
    bgl.clientCompilerItf = fgc;
    fgc.clientCompilerItf = bgc;

    fgc.definitionCompilerItf = definitionCompiler;

    bgl.compilerWrapperItf = compilerWrapper;
    bgl.outputFileLocatorItf = outputFileLocator;
    bgl.implementationLocatorItf = implementationLocator;
    bgc.definitionCompilerItf = definitionCompiler;
    bgc.instanceCompilerItf = instanceCompiler;

    return graphCompiler;
  }

  public static GraphCompiler newGraphCompiler() {
    final IDLLoader idlLoader = IDLLoaderChainFactory.newLoader();
    final BasicInputResourceLocator inputResourceLocator = new BasicInputResourceLocator();
    final BasicOutputFileLocator outputFileLocator = new BasicOutputFileLocator();
    final ImplementationLocator implementationLocator = new BasicImplementationLocator();

    final BasicASTTransformer astTransformer = new BasicASTTransformer();
    astTransformer.nodeFactoryItf = new STNodeFactoryImpl();

    final StringTemplateGroupLoader stcLoader = STLoaderFactory.newSTLoader();

    final IDLVisitor idlCompiler = IDLBackendFactory.newIDLCompiler(idlLoader,
        inputResourceLocator, outputFileLocator, astTransformer, stcLoader);

    final DefinitionSourceGenerator definitionSourceGenerator = newDefinitionSourceGenerator(
        inputResourceLocator, outputFileLocator, idlLoader, idlCompiler,
        astTransformer, stcLoader);
    final CompilerWrapper compilerWrapper = new GccCompilerWrapper();
    final MPPWrapper mppWrapper = new BasicMPPWrapper();

    final DefinitionCompiler definitionCompiler = newDefinitionCompiler(
        definitionSourceGenerator, implementationLocator, outputFileLocator,
        compilerWrapper, mppWrapper);

    return newGraphCompiler(inputResourceLocator, implementationLocator,
        outputFileLocator, compilerWrapper, mppWrapper, definitionCompiler,
        stcLoader);
  }

  public static CompilationCommandExecutor newCompilationCommandExecutor() {
    return new BasicCompilationCommandExecutor();
  }
}
